body{margin:0;font-family:-apple-system,BlinkMacSystemFont,"Segoe UI","Roboto","Oxygen","Ubuntu","Cantarell","Fira Sans","Droid Sans","Helvetica Neue",sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}code{font-family:source-code-pro,Menlo,Monaco,Consolas,"Courier New",monospace}*,:after,:before{box-sizing:border-box;font-weight:400}html.html{overflow-y:scroll}#body{flex:1 1;padding:0;margin:10px 0 0;font-size:14px;color:#333e48;display:flex;flex-direction:column;min-height:100vh;position:relative}#body a{text-decoration:none}#body a:hover{text-decoration:underline}table,td,th{border:1px solid #ccc;border-radius:2px}td,th{padding:5px}td{text-align:right}th{text-align:center}h1{font-size:28px}table.Flat,table.Flat td,table.Flat th{border:none;padding:3px 1px}.ModelSelector{margin:0 auto;text-align:center;text-align-last:center;border:none;padding:1px;font-size:28px;color:#333e48}.ModelSelector:hover{border:1px solid #ccc;border-radius:3px;padding:0}p{margin:5px auto}input,select{border:1px solid #ccc;border-radius:3px}input{padding:3px 6px}select{display:inline;width:auto;padding:2px}.MainSpace{display:flex;flex-flow:row;margin:0 auto 30px}.Filters{margin:0 auto;display:flex;align-items:center;flex-direction:column}.Filters p{margin:5px}input.FilterValue,select.FilterValue,select.Lookup{width:100%}.ToggleDiv,.ToggleLink{display:block}.ToggleDiv{padding-left:15px}.FieldsList{list-style-type:none;padding:0;margin:0 10px}button.Link{background:none!important;border:none;padding:0!important;cursor:pointer;color:#069;font-size:inherit;font:inherit}button.Link:hover{text-decoration:underline}div.Logo{position:absolute;margin:0 15px;font-size:30px;font-family:Impact,Charcoal,sans-serif;cursor:pointer}div.Index{margin:30px}div.Index>div{width:46%;float:left;font-size:20px;color:#333e48;border:1px solid #ccc;border-radius:3px;margin:2%;padding:0 2% 2%}div.Index button{display:block}
/*# sourceMappingURL=main.a3af6666.chunk.css.map */