# Generated by Django 2.0.13 on 2020-05-01 08:03

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("data_browser", "0003_remove_view_app"),
    ]

    operations = [
        migrations.RenameField(
            model_name="view", old_name="model", new_name="model_name",
        ),
    ]
